/************************************************************
 **
 ** bsc_cmd.h : command declarations
 **
 ** Copyright (c) 2004, Kyle A. York
 ** All rights reserved
 **
 ************************************************************/

#ifndef bsc_cmd_h__
#define bsc_cmd_h__

#include "bsc_parm.h"

/* intinsic functions (handled by the PIC code) */
typedef enum {
  bsc_intrinsic_type_null, /* unused */
  bsc_intrinsic_type_clear,
  bsc_intrinsic_type_end,
  bsc_intrinsic_type_delay,
  bsc_intrinsic_type_interrupt_enable,
  bsc_intrinsic_type_interrupt_disable,
  bsc_intrinsic_type_sleep,
  bsc_intrinsic_type_adc_config,
  bsc_intrinsic_type_adc_on,
  bsc_intrinsic_type_adc_off,
  bsc_intrinsic_type_adc_start,
  bsc_intrinsic_type_adc_store,
  bsc_intrinsic_type_adc_wait,
  bsc_intrinsic_type_pwm_config,
  bsc_intrinsic_type_pwm_duty,
  bsc_intrinsic_type_pwm_off,
  bsc_intrinsic_type_pwm_period,
  bsc_intrinsic_type_pwm_reg,
  bsc_intrinsic_type_interrupt_portb,
  bsc_intrinsic_type_pinhigh,
  bsc_intrinsic_type_pinlow,
  bsc_intrinsic_type_pinrd,
  bsc_intrinsic_type_portout,
  bsc_intrinsic_type_portrd,
  bsc_intrinsic_type_setport,
  bsc_intrinsic_type_rx_err_check,
  bsc_intrinsic_type_rx_store,
  bsc_intrinsic_type_tx_load,
  bsc_intrinsic_type_usart_config,
  bsc_intrinsic_type_wait_rx,
  bsc_intrinsic_type_wait_tx,
  /* timer */
  bsc_intrinsic_type_timer_config,
  bsc_intrinsic_type_timer_start,
  bsc_intrinsic_type_timer_stop,
  bsc_intrinsic_type_timer_countdown,
  bsc_intrinsic_type_timer_read,
  bsc_intrinsic_type_timer_write,
  /* eeprom */
  bsc_intrinsic_type_eeprom_write,
  bsc_intrinsic_type_eeprom_read,
  bsc_intrinsic_type_eeprom_wait
} bsc_intrinsic_type_t;

typedef enum {
  bsc_ir_type_null,
  bsc_ir_type_adc,
  bsc_ir_type_pwm,
  bsc_ir_type_rx,
  bsc_ir_type_tx,
  bsc_ir_type_timer,
  bsc_ir_type_rb,
  bsc_ir_type_user
} bsc_ir_type_t;

cmd_t *bsc_cmd_alloc(label_t *label,
  bsc_intrinsic_type_t op, size_t param_ct, const param_t *params);

bsc_intrinsic_type_t bsc_cmd_type_get(const cmd_t *cmd);
void                 bsc_cmd_type_set(cmd_t *cmd, bsc_intrinsic_type_t type);

size_t         bsc_cmd_paramct_get(const cmd_t *cmd);
const param_t *bsc_cmd_param_get(const cmd_t *cmd, size_t ii);

#ifdef MSDOS
#include "bsccmdd.h"
#endif

#endif /* bsc_cmd_h__ */

