/************************************************************
 **
 ** bsc_file.h : BSC file structure declarations
 **
 ** Copyright (c) 2004, Kyle A. York
 ** All rights reserved
 **
 ************************************************************/
#ifndef bsc_file_h__
#define bsc_file_h__

#include "pfile.h"
#include "label.h"

#define BSC_IR_NAME_SZ_MAX 32

/* these help find common errors */
typedef enum bsc_file_flag_ {
  bsc_file_flag_adc_config,       /* adc is configured     */
  bsc_file_flag_adc_start,        /* adc is started        */
  bsc_file_flag_interrupt_enable, /* interrupt_enable used */
  bsc_file_flag_timer_config,     /* timer is configured   */
  bsc_file_flag_timer_start,      /* timer is started      */
  bsc_file_flag_usart_config_rx,
  bsc_file_flag_usart_config_tx
} bsc_file_flag_t;

typedef unsigned bsc_pass_t;

/* isr types */
typedef enum bsc_isr_type_ {
  bsc_isr_type_first,
  bsc_isr_type_timer = bsc_isr_type_first, /* this MUST be first */
  bsc_isr_type_adc,
  bsc_isr_type_pwm,
  bsc_isr_type_rx,
  bsc_isr_type_tx,
  bsc_isr_type_rb,
  bsc_isr_type_user,
  bsc_isr_type_ct
} bsc_isr_type_t;

typedef struct bsc_file_ bsc_file_t;

result_t bsc_file_alloc(bsc_file_t **dst, pfile_t *pf);
void     bsc_file_free(bsc_file_t *bsc);
void     bsc_source_process(bsc_file_t *bsc);
void     bsc_file_process(bsc_file_t *bsc);

pfile_t *bsc_file_pf_get(bsc_file_t *bsc);
result_t bsc_file_isr_get(bsc_file_t *bf, bsc_isr_type_t isr, 
    boolean_t alloc, label_t **dst);
label_t *bsc_file_isr_next_get(bsc_file_t *bf, bsc_isr_type_t type);

void        bsc_file_pass_reset(bsc_file_t *bf);
bsc_pass_t  bsc_file_pass_get(bsc_file_t *bf);

size_t      bsc_file_strid_next(bsc_file_t *bsc);
void        bsc_file_log_unavail(bsc_file_t *bsc, const char *cmd);

void        bsc_file_include_path_set(bsc_file_t *bsc, const char *inc_path);
const char *bsc_file_include_path_get(const bsc_file_t *bsc);

boolean_t   bsc_file_flag_test(const bsc_file_t *bsc, bsc_file_flag_t flag);
void        bsc_file_flag_set(bsc_file_t *bsc, bsc_file_flag_t flag);
void        bsc_file_flag_clr(bsc_file_t *bsc, bsc_file_flag_t flag);

#ifdef MSDOS
#include "bsc_decl.h"
#endif

#endif /* bsc_file_h__ */

