/**********************************************************
 **
 ** cmd_br.c : the cmd_type_branch functions
 **
 ** Copyright (c) 2004, Kyle A. York
 ** All rights reserved
 **
 ***********************************************************/

#include "value.h"
#include "label.h"
#include "cmd.h"
#include "cmdbrdc.h"

static void cmd_branch_dump(cmd_t *cmd, FILE *dst)
{
  const char *tstr;
  const char *cstr;

  tstr = "{unknown}";
  switch (cmd_brtype_get(cmd)) {
    case cmd_branchtype_none:   tstr = "{none}"; break;
    case cmd_branchtype_goto:   tstr = "goto";   break;
    case cmd_branchtype_call:   tstr = "call";   break;
    case cmd_branchtype_return: tstr = "return"; break;
  }
  cstr = "{unknown}";
  switch (cmd_brcond_get(cmd)) {
    case cmd_branchcond_none:   cstr = 0 ;      break;
    case cmd_branchcond_true:   cstr = "true";  break;
    case cmd_branchcond_false:  cstr = "false"; break;
  }
  fprintf(dst, "%s ", tstr);
  if (cstr) {
    fprintf(dst, "%s ? ", cstr);
    value_dump(cmd_brval_get(cmd), dst);
  }
  if (cmd_brdst_get(cmd)) {
    if (cstr) {
      fprintf(dst, "-->");
    }
    fprintf(dst, "%s", label_name_get(cmd_brdst_get(cmd)));
  }
}

static void cmd_branch_free(cmd_t *cmd)
{
  cmd_brval_set(cmd, 0);
  cmd_brdst_set(cmd, 0);
}


static const cmd_vtbl_t cmd_br_vtbl = {
  cmd_branch_free,
  cmd_branch_dump
};

cmd_t *cmd_branch_alloc(label_t *label,
  cmd_branchtype_t type,
  cmd_branchcond_t cond, label_t *dst, value_t *var)
{
  cmd_branch_t *cbr;

  cbr = cmd_alloc(label, cmd_type_branch, &cmd_br_vtbl, sizeof(*cbr));
  if (cbr) {
    cbr->type = type;
    cbr->cond = cond;
    cbr->dst  = 0;
    cbr->var  = 0;
    
    cmd_brdst_set(&cbr->common, dst);
    cmd_brval_set(&cbr->common, var);
  }
  return &cbr->common;
}

cmd_branchtype_t cmd_brtype_get(const cmd_t *cmd)
{
  return (cmd_type_branch == cmd_type_get(cmd))
    ? ((const cmd_branch_t *) cmd)->type
    : cmd_branchtype_none;
}

void             cmd_brtype_set(cmd_t *cmd, cmd_branchtype_t brtype)
{
  if (cmd_type_branch == cmd_type_get(cmd)) {
    ((cmd_branch_t *) cmd)->type = brtype;
  }
}

cmd_branchcond_t cmd_brcond_get(const cmd_t *cmd)
{
  return (cmd_type_branch == cmd_type_get(cmd)) 
    ? ((const cmd_branch_t *) cmd)->cond
    : cmd_branchcond_none;
}

void             cmd_brcond_set(cmd_t *cmd, cmd_branchcond_t brcond)
{
  if (cmd_type_branch == cmd_type_get(cmd)) {
    ((cmd_branch_t *) cmd)->cond = brcond;
  }
}


label_t *cmd_brdst_get(cmd_t *cmd)
{
  return (cmd_type_branch == cmd_type_get(cmd))
    ? ((cmd_branch_t *) cmd)->dst
    : 0;
}

void              cmd_brdst_set(cmd_t *cmd, label_t *dst)
{
  if (cmd_type_branch == cmd_type_get(cmd)) {
    cmd_branch_t *cbr;

    cbr = (cmd_branch_t *) cmd;
    if (dst) {
      label_lock(dst);
      label_usage_bump(dst, ctr_bump_incr);
    }
    if (cbr->dst) {
      label_usage_bump(cbr->dst, ctr_bump_decr);
      label_release(cbr->dst);
    }
    cbr->dst = dst;
  }
}

value_t *cmd_brval_get(const cmd_t *cmd)
{
  return (cmd_type_branch == cmd_type_get(cmd)) 
    ? ((const cmd_branch_t *) cmd)->var
    : 0;
}

void              cmd_brval_set(cmd_t *cmd, value_t *val)
{
  if (cmd_type_branch == cmd_type_get(cmd)) {
    cmd_branch_t *cbr;

    cbr = (cmd_branch_t *) cmd;
    if (val) {
      value_lock(val);
      value_use_ct_bump(val, ctr_bump_incr);
    }
    if (cbr->var) {
      value_use_ct_bump(cbr->var, ctr_bump_decr);
      value_release(cbr->var);
    }
    cbr->var = val;
  }
}


