/************************************************************
 **
 ** label.h : label structure declarations
 **
 ** Copyright (c) 2004, Kyle A. York
 ** All rights reserved
 **
 ************************************************************/
#ifndef label_h__
#define label_h__

#include "cmd.h"

typedef struct {
  slist_t lst;
} lbllist_t;

typedef void (*label_free_cb_t)(label_t *lbl);
#define LABEL_FLAG_DEFINED 0x0001 /* this label has a definition   */
#define LABEL_FLAG_USED    0x0002 /* one external reference exists */

result_t label_alloc(label_t **dst, const char *name, label_free_cb_t cb);
void label_lock(label_t *lbl);
void label_release(label_t *lbl);

void label_list_init(lbllist_t *lst);
void label_list_reset(lbllist_t *lst, pfile_t *pf);
unsigned label_list_ct_get(const lbllist_t *lst);

typedef boolean_t (*lblfind_cb_t)(void *arg, const label_t *lbl, 
    const void *data);
label_t *label_list_find(lbllist_t *lst, lblfind_cb_t fn, void *arg,
  const void *data);
void label_list_append(lbllist_t *lst, label_t *lbl);


const char *label_name_get(const label_t *lbl);
void       *label_data_get(const label_t *lbl);
void        label_data_set(label_t *lbl, void *data);
boolean_t   label_flag_test(const label_t *lbl, flag_t flag);
void        label_flag_set(label_t *lbl,  flag_t flag);

ctr_t       label_usage_get(const label_t *lbl);
void        label_usage_bump(label_t *lbl, ctr_bump_t chg);

ctr_t       label_ref_ct_get(const label_t *lbl);

void        label_master_set(label_t *lbl, label_t *master);
label_t    *label_master_get(const label_t *lbl);

label_t    *label_next_get(label_t *lbl);
label_t    *label_list_head(lbllist_t *lst);

cmd_branchtype_t label_brtype_get(const label_t *lbl);
void             label_brtype_set(label_t *lbl, cmd_branchtype_t type);

label_t    *label_brdest_get(const label_t *lbl);
void        label_brdest_set(label_t *lbl, label_t *brdest);

size_t label_pc_get(label_t *lbl);
void   label_pc_set(label_t *lbl, size_t pc);

void        label_remove(label_t *lbl);

#ifdef MSDOS
#include "lbl_decl.h"
#endif

#endif /* label_h__ */

