/************************************************************
 **
 ** pic_bsc.c : PIC BSC specific functions
 **
 ** Copyright (c) 2004, Kyle A. York
 ** All rights reserved
 **
 ************************************************************/
#include "pfile.h"
#include "bsc_file.h"
#include "bsc_tokn.h"
#include "bsc_intr.h"
#include "bsc_expr.h"
#include "pic.h"
#include "pic_op.h"
#include "pic_bsc.h"

static void pic_txif_wait(pfile_t *pf)
{
  label_t *lbl;

  if (result_ok == pfile_label_alloc(pf, 0, 0, &lbl)) {
    pic_instr_append_label(pf, lbl);
    pic_instr_append_reg_flag(pf, pic_opcode_btfss, "_pir1", "_txif");
    pic_instr_append_n(pf, pic_opcode_goto, lbl);
    label_release(lbl);
  }
}

/*
 * NAME
 *   pic_printfn_create
 *
 * DESCRIPTION
 *   create the print function
 *
 * PARAMETERS
 *
 * RETURN
 *
 * NOTES
 *   This really belongs in a different file, possibly something like
 *   pic_bsc.c since it's specific to the basic language, but I'm going
 *   to stuff it here for now. The same functionality can be done in
 *   bsc_intr.c, but doing it here I can take advantage of the fact that
 *   I know the division is done in a function, and I also know that
 *   _accum4 will never change, _accum2 holds the result, and _accum3
 *   the remainder which saves a call to _div. It turns out that it
 *   only saves 7 instructions though!
 *     What I really need to do is keep track of what's in the accumulators
 *   so I can optimize calls to div, but that can come later  
 */
void pic_printfn_create(pfile_t *pf)
{
  label_t *sprint;
  label_t *print;

  sprint = pfile_label_find(pf, pfile_log_none, BSC_SPRINTFN);
  print  = pfile_label_find(pf, pfile_log_none, BSC_PRINTFN);
  if (print || sprint) {
    value_t *txreg;
    value_t *accum1;
    value_t *accum2;
    value_t *accum3;
    value_t *accum4;
    value_t *printbuf;
    value_t *printval;
    value_t *zero;

    if (sprint && !print) {
      pfile_label_alloc(pf, BSC_PRINTFN, 0, &print);
    }
    txreg  = pfile_value_find(pf, pfile_log_err, PFILE_LENGTH_DEF, "_txreg");
    accum1 = pic_accum_get(pf, 1, div_sz, 0);
    accum2 = pic_accum_get(pf, 2, div_sz, 0);
    accum3 = pic_accum_get(pf, 3, div_sz, 0);
    accum4 = pic_accum_get(pf, 4, div_sz, 0);
    printval = pfile_value_find(pf, pfile_log_err, PFILE_LENGTH_DEF, 
        BSC_PRINTVAL);
    zero = 0;
    pfile_constant_get(pf, '0', &zero);
    if (printval) {
      printbuf = pfile_value_find(pf, pfile_log_none, PFILE_LENGTH_DEF, 
          BSC_PRINTBUF);
      if (0 == printbuf) {
        /* there are ~3.32 decimal digits/bit, so this is how big the
         * print buffer must be */
        pfile_value_alloc(pf, BSC_PRINTBUF, 0, 1, 
            (value_sz_get(printval) * 80 + 32) / 33, 0, &printbuf);
      }
    }

    if (txreg && accum1 && accum2 && accum3 && accum4 && printbuf && zero
        && printval) {
      value_t *c;

      pic_instr_append_label(pf, sprint);
      if (sprint) {
        /* signed print. accum1 holds the value */
        /* if accum >= 0 goto print */
        if (result_ok == pfile_constant_get(pf, 8 * value_sz_get(printval) - 1,
            &c)) {
          value_t *tmp;

          if (result_ok == pfile_value_temp_get(pf, 1, 0, &tmp)) {
            pic_op(pf, operator_bittst, tmp, printval, c);
            pic_branch(pf, cmd_branchtype_goto, cmd_branchcond_false, print,
                tmp);
            value_release(tmp);
          }
          value_release(c);
        }
        /* output '-' and negate accum1 */
        if (result_ok == pfile_constant_get(pf, '-', &c)) {
          pic_txif_wait(pf);
          pic_op(pf, operator_assign, txreg, c, 0);
          value_release(c);
        }
        /* negate print_val */
        pic_op(pf, operator_neg, printval, 0, 0);
        /* this falls through the print */
      }
      pic_instr_append_label(pf, print);
      if (result_ok == pfile_constant_get(pf, 10, &c)) {
        label_t *top;
        value_t *tmp;

        pic_op(pf, operator_assign, accum1, printval, 0);
        pic_op(pf, operator_assign, accum4, c, 0);
        value_release(c);
        /* do { 
             printbuf[tmp++] = accum % 10; accum /= 10; 
           } while (accum) */
        c = 0;
        tmp = 0;
        top = 0;
        
        pfile_constant_get(pf, 0, &c);
        pfile_value_temp_get(pf, 1, 0, &tmp);
        pfile_label_alloc(pf, 0, 0, &top);
        if (c && tmp && top) {
          pic_op(pf, operator_assign, tmp, c, 0);
          value_subscript_set(printbuf, value_variable_get(tmp));
          pic_instr_append_label(pf, top);
          pic_op(pf, operator_div, accum1, accum4, 0);
          pic_op(pf, operator_assign, printbuf, accum2, 0);
          pic_instr_append_w_k(pf, pic_opcode_movlw, zero);
          pic_instr_append_reg_d(pf, pic_opcode_addwf, "_ind", pic_opdst_f);
          pic_op(pf, operator_incr, tmp, 0, 0);
          pic_branch(pf, cmd_branchtype_goto, cmd_branchcond_true, top, 
              accum1);
          /* while (tmp--) {
           *   print printbuf[tmp]
           * }
           */ 
          label_release(top);
          top = 0;
          if (result_ok == pfile_label_alloc(pf, 0, 0, &top)) {
            pic_instr_append_label(pf, top);
            pic_txif_wait(pf);
            pic_op(pf, operator_decr, tmp, 0, 0);
            pic_op(pf, operator_assign, txreg, printbuf, 0);
            pic_branch(pf, cmd_branchtype_goto, 
              cmd_branchcond_true, top, tmp);
            pic_branch(pf, cmd_branchtype_return, cmd_branchcond_none, 0, 0);
          }
        }
        label_release(top);
        value_release(tmp);
      }
      value_release(c);
    }
    value_release(printval);
    value_release(zero);
    value_release(printbuf);
    value_release(accum4);
    value_release(accum3);
    value_release(accum2);
    value_release(accum1);
    value_release(txreg);
    label_release(sprint);
    label_release(print);
  }
}

static void pic_parse_pragma_num(bsc_file_t *bsc, value_t **dst)
{
  bsc_token_get(bsc, bsc_token_next);
  *dst = 0;
  bsc_expr_parse(bsc, 0, dst);
  if (!value_is_const(*dst)) {
    pfile_log(bsc_file_pf_get(bsc), pfile_log_err, BSC_MSG_CONSTANT_EXPECTED);
    value_release(*dst);
    *dst = 0;
  }
}

static void pic_parse_pragma_range(bsc_file_t *bsc, value_t **lo, value_t **hi)
{
  *hi = 0;
  pic_parse_pragma_num(bsc, lo);
  if (*lo) {
    if (!bsc_token_is(bsc, bsc_token_current, pfile_log_none, ",")) {
      *hi = *lo;
      value_lock(*hi);
    } else {
      pic_parse_pragma_num(bsc, hi);
      if (!*hi) {
        value_release(*lo);
        *lo = 0;
      } else if (value_const_get(*lo) > value_const_get(*hi)) {
        pfile_log(bsc_file_pf_get(bsc), pfile_log_err, PIC_MSG_RANGE_INVALID);
        value_release(*lo);
        value_release(*hi);
        *lo = 0;
        *hi = 0;
      }
    }
  }
}

void pic_parse_pragma(bsc_file_t *bsc)
{
  pfile_t *pf;

  pf = bsc_file_pf_get(bsc);

  if (bsc_token_is(bsc, bsc_token_current, pfile_log_none, "data")) {
    /* lo, high */
    value_t *lo;
    value_t *hi;

    pic_parse_pragma_range(bsc, &lo, &hi);
    if (lo && hi) {
      pic_blist_add(pf, &pic_blist, (size_t) value_const_get(lo), 
        (size_t) value_const_get(hi));
      value_release(hi);
      value_release(lo);
    }
  } else if (bsc_token_is(bsc, bsc_token_current, pfile_log_none, "restart")) {
    /* loc */
    value_t *loc;

    pic_parse_pragma_num(bsc, &loc);
    if (loc) {
      value_release(loc);
    }
  } else if (bsc_token_is(bsc, bsc_token_current, pfile_log_none, "interrupt")) {
    /* loc */
    value_t *loc;

    pic_parse_pragma_num(bsc, &loc);
    if (loc) {
      value_release(loc);
    }
  } else if (bsc_token_is(bsc, bsc_token_current, pfile_log_none, "code")) {
    /* size */
    value_t *loc;

    pic_parse_pragma_num(bsc, &loc);
    if (loc) {
      pic_mem_size = (size_t) value_const_get(loc);
      value_release(loc);
    }
  } else if (bsc_token_is(bsc, bsc_token_current, pfile_log_none, "picloader")) {
    /* loc */
    value_t *loc;

    pic_parse_pragma_num(bsc, &loc);
    if (loc) {
      if (value_const_get(loc) < pic_mem_size) {
        pic_mem_size_loader = (size_t) value_const_get(loc);
      } else {
        pfile_log(pf, pfile_log_err, PIC_MSG_PICLOADER_START_BAD);
      }
      value_release(loc);
    }
  } else {
    pfile_log(pf, pfile_log_err, PIC_MSG_PRAGMA_UNKNOWN, 
        bsc_token_get(bsc, bsc_token_current));
  }
}

