/************************************************************
 **
 ** pic_inst.h : PIC instructions declarations
 **
 ** Copyright (c) 2004, Kyle A. York
 ** All rights reserved
 **
 ************************************************************/
#ifndef pic_inst_h__
#define pic_inst_h__

#include "pfile.h"

/* pic instruction generation */

/* d = W or F                */
/* f = file register         */
/* b = bit (0..7)            */
/* k = constant (0..255)     */
/* n = destination (0..2047) */
typedef enum {
  pic_opcode_org,    /* ORG xxxx */
  pic_opcode_end,    /* END      */
  pic_opcode_none,   /* no code here */
  /* pic_instr_append_f_d   */
  /* pic_instr_append_reg_d */
  pic_opcode_addwf,  /* f  + w --> d                 */
  pic_opcode_andwf,  /* f  & w --> d                 */
  pic_opcode_xorwf,  /* f  ^ w --> d                 */
  pic_opcode_iorwf,  /* f  | w --> d                 */
  pic_opcode_subwf,  /* f  - w --> d                 */
  pic_opcode_comf,   /* ~f     --> d                 */
  pic_opcode_decf,   /* f - 1  --> d                 */
  pic_opcode_decfsz, /* f - 1  --> d, skip next if 0 */
  pic_opcode_incf,   /* f + 1  --> d                 */
  pic_opcode_incfsz, /* f + 1  --> d, skip next if 0 */
  pic_opcode_rlf,    /* f << 1 --> d                 */
  pic_opcode_rrf,    /* f >> 1 --> d                 */
  pic_opcode_movf,   /* f      --> d                 */
  pic_opcode_swapf,  /* swap nibbles in f --> d      */
  /* pic_instr_append_f   */
  /* pic_instr_append_reg */
  pic_opcode_clrf,   /* f = 0                        */
  pic_opcode_movwf,  /* w      --> f                 */
  /* pic_instr_append */
  pic_opcode_nop,    /*                              */
  pic_opcode_clrw,   /* w = 0     */
  pic_opcode_retfie, /* return from interrupt */
  pic_opcode_ret,    /* return */
  pic_opcode_sleep,
  pic_opcode_clrwdt,
  /* pic_instr_append_f_b      */
  /* pic_instr_append_reg_flag */
  /* pic_instr_append_f_bn     */
  /* pic_instr_append_reg_bn   */
  pic_opcode_bcf,    /* f, b -- bit clear */
  pic_opcode_bsf,    /* f, b -- bit set   */
  pic_opcode_btfsc,  /* f, b -- bit test, skip next if clear */
  pic_opcode_btfss,  /* f, b -- bit test, skip next if set   */
  /* pic_instr_append_w_k  */
  /* pic_instr_append_w_kn */
  pic_opcode_addlw,  /* w = k + w */
  pic_opcode_andlw,  /* w = k & w */
  pic_opcode_iorlw,  /* w = k | w */
  pic_opcode_movlw,  /* w = k     */
  pic_opcode_sublw,  /* w = k - w */
  pic_opcode_xorlw,  /* w = k ^ w */
  pic_opcode_retlw,
  /* use pic_instr_append_n */
  pic_opcode_call,   /* call n */
  pic_opcode_goto,   /* goto n */
  /* pseudo-ops to make life easier, translation in comments */
  pic_opcode_datalo_set, /* bsf _status, _rp0 */
  pic_opcode_datalo_clr, /* bcf _status, _rp0 */
  pic_opcode_datahi_set, /* bsf _status, _rp1 */
  pic_opcode_datahi_clr, /* bcf _status, _rp1 */
  pic_opcode_branchlo_set, /* bsf _pclath, 3 */
  pic_opcode_branchlo_clr, /* bcf _pclath, 3 */
  pic_opcode_branchhi_set, /* bsf _pclath, 4 */
  pic_opcode_branchhi_clr  /* bcf _pclath, 4 */
} pic_opcode_t;

typedef enum {
  pic_opdst_none, /* no dst */
  pic_opdst_w,
  pic_opdst_f
} pic_opdst_t;

void pic_instr_append_label(pfile_t *pf, label_t *lbl);
void pic_instr_append_f_d(pfile_t *pf, pic_opcode_t op, value_t *f, 
    size_t ofs, pic_opdst_t dst);
void pic_instr_append_reg_d(pfile_t *pf, pic_opcode_t op, const char *reg,
  pic_opdst_t dst);
void pic_instr_append_f(pfile_t *pf, pic_opcode_t op, value_t *f, 
    size_t ofs);
void pic_instr_append_reg(pfile_t *pf, pic_opcode_t op, const char *reg);
void pic_instr_append(pfile_t *pf, pic_opcode_t op);
void pic_instr_append_f_b(pfile_t *pf, pic_opcode_t op, value_t *f, 
    size_t ofs, value_t *bit);
void pic_instr_append_reg_bn(pfile_t *pf, pic_opcode_t op, 
    const char *reg, variable_const_t bit);
void pic_instr_append_reg_flag(pfile_t *pf, pic_opcode_t op, 
    const char *reg, const char *flag);
void pic_instr_append_f_bn(pfile_t *pf, pic_opcode_t op,
    value_t *f, size_t ofs, variable_const_t n);
void pic_instr_append_w_k(pfile_t *pf, pic_opcode_t op, 
    value_t *k);
void pic_instr_append_w_kn(pfile_t *pf, pic_opcode_t op,
    variable_const_t n);
void pic_instr_append_n(pfile_t *pf, pic_opcode_t op, label_t *dst);


#endif /* pic_inst_h__ */

